-----------------------------------------------------
-- lua 파일이 여러개인 경우
-- 애드온명, 애드온내공유테이블이 ...으로 전달됨
-----------------------------------------------------
local addonName, L = ...;

--------------------------------------------------------------------------
-- 20191111
-- 아이템 툴팁이 화면밖으로 나가는 현상 처리
-- IsClampedToScreen() 값이 FALSE(화면밖이동허용) 이면, 다시 복구처리함.
-- 확인 1) 이 현상은 전투중 몹의 버프, 디버프 툴팁을 확인했을때 나타남.
--       ==> 툴팁을 띄우는 방법 확인 필요
-- 진득추측)
-- 이름표는 언제든 화면을 벗어날수 있다.(화면 돌림등.)
-- 하지만, 화면내 고정하는 옵션이 문제될수 있어서 제외한듯.(8.2.5쯤 부터)
-- 이름표뿐 아니라 툴팁이 화면을 벗어날수 있는 Widget이 많으므로..
--------------------------------------------------------------------------
function on_GameTooltip_Show(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);
    
    if( not self:IsClampedToScreen() ) then
        --print( self:GetName().."[|cff000000tooltip:IsClampedToScreen change to : "..tostring(self:IsClampedToScreen()).."|r]" );
        ------------------------------------------------------------------------------------------
        -- 20191129
        -- self:SetClampedToScreen(true); 
        -- --> Action[ClampedToScreen] failed because[Can't clamp restricted regions] 오류발생함.
        -- nameplate 특성상 고정되어 있지 않아서 인듯.
        -- lua의 try catch문, pcall 이용하여 pass
        ------------------------------------------------------------------------------------------
        -- self:SetClampedToScreen(true); 
        local status, retval = pcall(self.SetClampedToScreen, self, true);
        --print(status);
        --print(retval);
    end
end
GameTooltip:HookScript("OnShow",on_GameTooltip_Show);

-- 20190927
-- Action 버튼, 케릭터 등 화면하단에 고정된 툴팁 위치 이동
-- (참조 asTooltip)
function on_GameTooltip_SetDefaultAnchor(tooltip, parent,...)
    -- Return if no tooltip or parent
    if (not tooltip or not parent) then
        return;
    end

    -- 20190927
    -- 툴팁이동설정시에만 기능동작하도록 수정함.
    if not LJDSpellInterruptDB["ReLocateGameToolTip"] then
        return;
    end

    local AFFL_X = 385;                  --툴팁 X 위치

    tooltip:SetOwner(parent, "ANCHOR_NONE");
    tooltip:ClearAllPoints();
    tooltip:SetPoint("BOTTOM", "UIParent", "BOTTOM", AFFL_X, getTooltipBottomPosY());
    tooltip:SetScale(0.9);
end
hooksecurefunc("GameTooltip_SetDefaultAnchor", on_GameTooltip_SetDefaultAnchor);

--------------------------------------------------------------------------
-- 20191114
-- 버프/디버프 아이콘 툴팁에도 시전자 표시
--------------------------------------------------------------------------
function on_GameTooltip_SetUnitAura(self, ...)
    local casterId = select(7, UnitAura(...));
    --DebugPrint(casterId);
    local unitId, auraIndex, filter = ...;

    if casterId then

        if not UnitIsUnit(casterId, unitId) then
            local className, classFilename, classID = UnitClass(casterId);
            local rPerc, gPerc, bPerc, argbHex = GetClassColor(classFilename);
            local casterName = UnitName(casterId);

            self:AddLine(" ");
            self:AddDoubleLine(L["시전"]..":", casterName, _, _, _, rPerc, gPerc, bPerc);
            self:Show();
        end
    end
end
hooksecurefunc(GameTooltip, "SetUnitAura", on_GameTooltip_SetUnitAura);

--------------------------------------------------------------------------
-- 20191111
-- 아이템, 스킬, 퀘스트 참조 툴팁(채팅창의 링크 클릭해서 열리는)이 
-- 스킬창을 가리는 것을 해결하기 위해 위치 조정.
--------------------------------------------------------------------------
function on_ItemRefTooltip_Show(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);

    -- 툴팁이동설정시에만 기능동작하도록 수정함.
    if not LJDSpellInterruptDB["ReLocateGameToolTip"] then
        return
    end

    --self:SetOwner(self:GetParent(), "ANCHOR_NONE");   -- 특이점.. 이 라인 주석해제시 툴팁이 나오지 않음.
    self:ClearAllPoints();
    self:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, getTooltipBottomPosY());
end
ItemRefTooltip:HookScript("OnShow", on_ItemRefTooltip_Show);

--------------------------------------------------------------------------
-- 20191111
-- 아이템 툴팁의 전체 보유량 표시
--------------------------------------------------------------------------
function on_GameTooltip_SetItem(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);
    local itemName, itemLink = self:GetItem();
    local itemCount = GetItemCount(itemLink, true);   -- 은행포함.(2번째 인자)
    
    --print(itemName..":"..itemCount);
    if itemName == '' then
        return;
    end
    
    self:AddLine("OnTooltipSetItem");
    self:AddDoubleLine(L["보유"]..":", itemCount, _, _, _, 1, 1, 1);
    self:Show();
            
end
--GameTooltip:HookScript("OnTooltipSetItem", on_GameTooltip_SetItem);

-- 재료템
function on_GameTooltip_SetRecipeReagentItem(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);
    local recipe, reagent = ...;
    --print(C_TradeSkillUI.GetRecipeReagentItemLink(recipe, reagent));
    
    local itemLink = C_TradeSkillUI.GetRecipeReagentItemLink(recipe, reagent);
    local itemCount = GetItemCount(itemLink, true);   -- 은행포함.(2번째 인자)
    --print(itemLink..":"..itemCount);

    self:AddLine("SetRecipeReagentItem");
    self:AddDoubleLine(L["보유"]..":", itemCount, _, _, _, 1, 1, 1);
    self:Show();

end
--hooksecurefunc(GameTooltip, 'SetRecipeReagentItem', on_GameTooltip_SetRecipeReagentItem)

-- 결과템
function on_GameTooltip_SetRecipeResultItem(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);
    local recipe = ...;
    --print(C_TradeSkillUI.GetRecipeItemLink(recipe));
    
    local itemLink = C_TradeSkillUI.GetRecipeItemLink(recipe);
    local itemCount = GetItemCount(itemLink, true);   -- 은행포함.(2번째 인자)
    --print(itemLink..":"..itemCount);
    
    self:AddLine("SetRecipeResultItem");
    self:AddDoubleLine(L["보유"]..":", itemCount, _, _, _, 1, 1, 1);
    self:Show();
end
--hooksecurefunc(GameTooltip, 'SetRecipeResultItem', on_GameTooltip_SetRecipeResultItem);

-- 가방열었을때, 아이템 링크창의 아이콘을 깜박임 표시한다.
-- ;금방 습득한 아이템을 채팅창의 링크를 클릭했을때, 가방의 어디에 있는지 쉽게 찾기 기능.
function on_ContainerFrame_OnShow(self)

    -- 1초마다 재귀호출
    if not self.LJDGameTooltipTicker then
        self.LJDGameTooltipTicker = C_Timer.NewTicker(0.5, function() on_ContainerFrame_OnShow(self) end );
    end

    --DebugPrint(self:GetName());
    local tarItemName, tarItemLink = GameTooltip:GetItem();
    local refItemName, refItemLink = ItemRefTooltip:GetItem();
    
    -- 우선순위 : 아이템링크 1순위
    if( refItemLink ) then
        tarItemLink = refItemLink;
    end

    --print(tarItemLink);
    -- 아이템정보 없으면 패쓰
    if( not tarItemLink ) then
        -- 디버그용
        --tarItemLink = "|cffffffff|Hitem:6948::::::::120:66::::::|h[귀환석]|h|r";
        tarItemLink = "|cffffffff|Hitem:158379::::::::120:66::13:1:3524:::|h[불안정한 시간 변형기]|h|r";
        return;
    end
    
    -- 링크 복사용
    --DebugPrintLinkError(tarItemLink);

    local tarItemID = string.match(tarItemLink, "item:(%d+)");  -- 링크에서 ID 추출
    tarItemID = tonumber(tarItemID);
    --DebugPrint(tarItemID);
    
    -- 모든 가방 확인할때 for(여기서는 가방마다 함수가 호출되므로 필요없음)
    --for bagID = 0, NUM_BAG_SLOTS do
    --end
    
    local bagID = self:GetID();
    -- 모든 슬롯
    for slot = 1, GetContainerNumSlots(bagID) do
            
        --print(bagID, slot);
        -------------------------------------------------------------
        -- 특이점
        -- 버튼의 GetID() 와 ButtonName index, slot는 일치하지 않음
        -- ContainerFrame.lua를 참조하여
        -- 버튼의 GetID() 이용함.
        -------------------------------------------------------------
        local itemButtonName = (self:GetName()).."Item"..slot;
        local itemButton = _G[itemButtonName];
                
        -- 각 가방의 슬롯, 아이템 확인
        local icon, itemCount, locked, quality, readable, lootable, itemLink, isFiltered, noValue, itemID = GetContainerItemInfo(bagID, itemButton:GetID());
        --print(itemButtonName, itemLink, bagID, slot, itemButton:GetID());
        --print(tarItemID, itemID);
        --DebugPrint({GetContainerItemInfo(bagID, slot)});

        --DebugPrintLink(tarItemLink);
        --DebugPrintLink(itemLink);
        
        if( itemButton.flashAnim:IsPlaying() or itemButton.newitemglowAnim:IsPlaying()) then
            itemButton.flashAnim:Stop();
            itemButton.newitemglowAnim:Stop();
        end
                
        -- 아이템링크와 같은 아이템이면 눈에 띄게 표현
        if( tarItemID and tarItemID == itemID ) then
            --print(tarItemID, itemID);
            --DebugPrint("아이템 발견");
            
            -- 아이템위에 마우스르 올린 경우 깜박임 처리 안함
            if ( not GameTooltip:IsOwned(itemButton) ) then
                if (not itemButton.flashAnim:IsPlaying() and not itemButton.newitemglowAnim:IsPlaying()) then
                    itemButton.flashAnim:Play();
                    itemButton.newitemglowAnim:Play();
                end
            end
            
            --[[
            local btnTexture = itemButton.Icon or itemButton.icon or _G[itemButton:GetName().."IconTexture"];
            local btnColR, btnColG, btnColB, btnColA = btnTexture:GetVertexColor();

            -- 1 으로 설정해도 1 보다 모자르다. ex) 0.99999779462814
            if( btnColR > 0.9999 ) then btnColR = 1 end;
            if( btnColG > 0.9999 ) then btnColG = 1 end;
            if( btnColB > 0.9999 ) then btnColB = 1 end;
            if( btnColA > 0.9999 ) then btnColA = 1 end;

            --SetItemButtonTextureVertexColor(itemButton, 1,0,0);
            if( btnColR == 1 and btnColG == 0 and btnColB == 0 ) then
                --btnTexture:SetVertexColor(1, 1, 1);
            else
                --btnTexture:SetVertexColor(1, 0, 0);
            end
            
            --itemButton.IconBorder:SetVertexColor(1, 0, 0);
            --]]
        end
    end

end
hooksecurefunc("ContainerFrame_OnShow", on_ContainerFrame_OnShow);

function on_ContainerFrame_OnHide(self)
    --print("Hide");
    -- 타이머 중지 및 제거
    if self.LJDGameTooltipTicker then
        self.LJDGameTooltipTicker:Cancel();
        self.LJDGameTooltipTicker = nil;
    end
end
hooksecurefunc("ContainerFrame_OnHide", on_ContainerFrame_OnHide);
